@@ -69,6 +69,8 @@ public:
     const AndroidSdkPackageList &allSdkPackages();
     AndroidSdkPackageList availableSdkPackages();
     AndroidSdkPackageList installedSdkPackages();
+    SystemImageList installedSystemImages();
+    NdkList installedNdkPackages();
 
     SdkPlatform *latestAndroidSdkPlatform(AndroidSdkPackage::PackageState state
                                           = AndroidSdkPackage::Installed);
@@ -78,6 +80,8 @@ public:
     void reloadPackages(bool forceReload = false);
     bool isBusy() const;
 
+    bool packageListingSuccessful() const;
+
     QFuture<QString> availableArguments() const;
     QFuture<OperationOutput> updateAll();
     QFuture<OperationOutput> update(const QStringList &install, const QStringList &uninstall);
