@@ -35,17 +35,21 @@
 #include <projectexplorer/project.h>
 #include <projectexplorer/buildsteplist.h>
 #include <projectexplorer/kitinformation.h>
+#include <projectexplorer/processparameters.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/buildconfiguration.h>
 #include <projectexplorer/projectexplorerconstants.h>
+
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtparser.h>
+
 #include <utils/stringutils.h>
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
 
 using namespace Core;
 using namespace ProjectExplorer;
+using namespace Utils;
 
 namespace Ios {
 namespace Internal {
@@ -55,27 +59,23 @@ static const char COMMAND_PARTIAL_KEY[] = ".Command";
 static const char ARGUMENTS_PARTIAL_KEY[] = ".Arguments";
 static const char CLEAN_PARTIAL_KEY[] = ".Clean";
 
-IosDsymBuildStep::IosDsymBuildStep(BuildStepList *parent) :
-    AbstractProcessStep(parent, Constants::IOS_DSYM_BUILD_STEP_ID),
+IosDsymBuildStep::IosDsymBuildStep(BuildStepList *parent, Id id) :
+    AbstractProcessStep(parent, id),
     m_clean(parent->id() == ProjectExplorer::Constants::BUILDSTEPS_CLEAN)
 {
 }
 
-bool IosDsymBuildStep::init(QList<const BuildStep *> &earlierSteps)
+bool IosDsymBuildStep::init()
 {
     BuildConfiguration *bc = buildConfiguration();
-    if (!bc)
-        bc = target()->activeBuildConfiguration();
 
     ProcessParameters *pp = processParameters();
     pp->setMacroExpander(bc->macroExpander());
-    pp->setWorkingDirectory(bc->buildDirectory().toString());
+    pp->setWorkingDirectory(bc->buildDirectory());
     Utils::Environment env = bc->environment();
     Utils::Environment::setupEnglishOutput(&env);
     pp->setEnvironment(env);
-    pp->setCommand(command());
-    pp->setArguments(Utils::QtcProcess::joinArgs(arguments()));
-    pp->resolveAll();
+    pp->setCommandLine({command(), arguments()});
 
     // If we are cleaning, then build can fail with an error code, but that doesn't mean
     // we should stop the clean queue
@@ -86,7 +86,7 @@ bool IosDsymBuildStep::init(QList<const BuildStep *> &earlierSteps)
     if (outputParser())
         outputParser()->setWorkingDirectory(pp->effectiveWorkingDirectory());
 
-    return AbstractProcessStep::init(earlierSteps);
+    return AbstractProcessStep::init();
 }
 
 QVariantMap IosDsymBuildStep::toMap() const
@@ -98,7 +98,7 @@ QVariantMap IosDsymBuildStep::toMap() const
     map.insert(id().withSuffix(USE_DEFAULT_ARGS_PARTIAL_KEY).toString(),
                isDefault());
     map.insert(id().withSuffix(CLEAN_PARTIAL_KEY).toString(), m_clean);
-    map.insert(id().withSuffix(COMMAND_PARTIAL_KEY).toString(), command());
+    map.insert(id().withSuffix(COMMAND_PARTIAL_KEY).toString(), command().toVariant());
     return map;
 }
 
@@ -109,8 +109,7 @@ bool IosDsymBuildStep::fromMap(const QVariantMap &map)
     bool useDefaultArguments = map.value(
                 id().withSuffix(USE_DEFAULT_ARGS_PARTIAL_KEY).toString()).toBool();
     m_clean = map.value(id().withSuffix(CLEAN_PARTIAL_KEY).toString(), m_clean).toBool();
-    m_command = map.value(id().withSuffix(COMMAND_PARTIAL_KEY).toString(), m_command)
-            .toString();
+    m_command = FilePath::fromVariant(map.value(id().withSuffix(COMMAND_PARTIAL_KEY).toString()));
     if (useDefaultArguments) {
         m_command = defaultCommand();
         m_arguments = defaultArguments();
@@ -126,12 +125,12 @@ QStringList IosDsymBuildStep::defaultArguments() const
     return defaultCmdList().mid(1);
 }
 
-QString IosDsymBuildStep::defaultCommand() const
+FilePath IosDsymBuildStep::defaultCommand() const
 {
     if (m_clean)
-        return defaultCleanCmdList().at(0);
+        return FilePath::fromString(defaultCleanCmdList().at(0));
     else
-        return defaultCmdList().at(0);
+        return FilePath::fromString(defaultCmdList().at(0));
 }
 
 QStringList IosDsymBuildStep::defaultCleanCmdList() const
@@ -147,12 +146,11 @@ QStringList IosDsymBuildStep::defaultCleanCmdList() const
 QStringList IosDsymBuildStep::defaultCmdList() const
 {
     QString dsymutilCmd = "dsymutil";
-    Utils::FileName dsymUtilPath = IosConfigurations::developerPath()
-            .appendPath("Toolchains/XcodeDefault.xctoolchain/usr/bin/dsymutil");
+    const Utils::FilePath dsymUtilPath = IosConfigurations::developerPath()
+            .pathAppended("Toolchains/XcodeDefault.xctoolchain/usr/bin/dsymutil");
     if (dsymUtilPath.exists())
         dsymutilCmd = dsymUtilPath.toUserOutput();
-    IosRunConfiguration *runConf =
-            qobject_cast<IosRunConfiguration *>(target()->activeRunConfiguration());
+    auto runConf = qobject_cast<const IosRunConfiguration *>(target()->activeRunConfiguration());
     QTC_ASSERT(runConf, return QStringList("echo"));
     QString dsymPath = runConf->bundleDirectory().toUserOutput();
     dsymPath.chop(4);
@@ -160,14 +158,14 @@ QStringList IosDsymBuildStep::defaultCmdList() const
     return QStringList({dsymutilCmd, "-o", dsymPath, runConf->localExecutable().toUserOutput()});
 }
 
-QString IosDsymBuildStep::command() const
+FilePath IosDsymBuildStep::command() const
 {
     if (m_command.isEmpty())
         return defaultCommand();
     return m_command;
 }
 
-void IosDsymBuildStep::setCommand(const QString &command)
+void IosDsymBuildStep::setCommand(const FilePath &command)
 {
     if (command == m_command)
         return;
@@ -189,9 +187,9 @@ bool IosDsymBuildStep::isDefault() const
     return arguments() == defaultArguments() && command() == defaultCommand();
 }
 
-void IosDsymBuildStep::run(QFutureInterface<bool> &fi)
+void IosDsymBuildStep::doRun()
 {
-    AbstractProcessStep::run(fi);
+    AbstractProcessStep::doRun();
 }
 
 BuildStepConfigWidget *IosDsymBuildStep::createConfigWidget()
@@ -199,11 +197,6 @@ BuildStepConfigWidget *IosDsymBuildStep::createConfigWidget()
     return new IosDsymBuildStepConfigWidget(this);
 }
 
-bool IosDsymBuildStep::immutable() const
-{
-    return false;
-}
-
 void IosDsymBuildStep::setArguments(const QStringList &args)
 {
     if (arguments() == args)
@@ -229,14 +222,12 @@ QStringList IosDsymBuildStep::arguments() const
 //
 
 IosDsymBuildStepConfigWidget::IosDsymBuildStepConfigWidget(IosDsymBuildStep *buildStep)
-    : m_buildStep(buildStep)
+    : BuildStepConfigWidget(buildStep), m_buildStep(buildStep)
 {
     m_ui = new Ui::IosPresetBuildStep;
     m_ui->setupUi(this);
 
-    Project *pro = m_buildStep->target()->project();
-
-    m_ui->commandLineEdit->setText(m_buildStep->command());
+    m_ui->commandLineEdit->setText(m_buildStep->command().toString());
     m_ui->argumentsTextEdit->setPlainText(Utils::QtcProcess::joinArgs(
                                                    m_buildStep->arguments()));
     m_ui->resetDefaultsButton->setEnabled(!m_buildStep->isDefault());
@@ -253,15 +244,8 @@ IosDsymBuildStepConfigWidget::IosDsymBuildStepConfigWidget(IosDsymBuildStep *bui
             this, &IosDsymBuildStepConfigWidget::updateDetails);
     connect(m_buildStep->target(), &Target::kitChanged,
             this, &IosDsymBuildStepConfigWidget::updateDetails);
-    pro->subscribeSignal(&BuildConfiguration::environmentChanged, this, [this]() {
-        if (static_cast<BuildConfiguration *>(sender())->isActive())
-            updateDetails();
-    });
-    connect(pro, &Project::activeProjectConfigurationChanged,
-            this, [this](ProjectConfiguration *pc) {
-        if (pc && pc->isActive())
-            updateDetails();
-    });
+    connect(m_buildStep->buildConfiguration(), &BuildConfiguration::enabledChanged,
+            this, &IosDsymBuildStepConfigWidget::updateDetails);
 }
 
 IosDsymBuildStepConfigWidget::~IosDsymBuildStepConfigWidget()
@@ -269,35 +253,22 @@ IosDsymBuildStepConfigWidget::~IosDsymBuildStepConfigWidget()
     delete m_ui;
 }
 
-QString IosDsymBuildStepConfigWidget::displayName() const
-{
-    return m_buildStep->displayName();
-}
-
 void IosDsymBuildStepConfigWidget::updateDetails()
 {
     BuildConfiguration *bc = m_buildStep->buildConfiguration();
-    if (!bc)
-        bc = m_buildStep->target()->activeBuildConfiguration();
 
     ProcessParameters param;
     param.setMacroExpander(bc->macroExpander());
-    param.setWorkingDirectory(bc->buildDirectory().toString());
+    param.setWorkingDirectory(bc->buildDirectory());
     param.setEnvironment(bc->environment());
-    param.setCommand(m_buildStep->command());
-    param.setArguments(Utils::QtcProcess::joinArgs(m_buildStep->arguments()));
-    m_summaryText = param.summary(displayName());
-    emit updateSummary();
-}
+    param.setCommandLine({m_buildStep->command(), m_buildStep->arguments()});
 
-QString IosDsymBuildStepConfigWidget::summaryText() const
-{
-    return m_summaryText;
+    setSummaryText(param.summary(displayName()));
 }
 
 void IosDsymBuildStepConfigWidget::commandChanged()
 {
-    m_buildStep->setCommand(m_ui->commandLineEdit->text());
+    m_buildStep->setCommand(FilePath::fromString(m_ui->commandLineEdit->text()));
     m_ui->resetDefaultsButton->setEnabled(!m_buildStep->isDefault());
     updateDetails();
 }
@@ -314,7 +285,7 @@ void IosDsymBuildStepConfigWidget::resetDefaults()
 {
     m_buildStep->setCommand(m_buildStep->defaultCommand());
     m_buildStep->setArguments(m_buildStep->defaultArguments());
-    m_ui->commandLineEdit->setText(m_buildStep->command());
+    m_ui->commandLineEdit->setText(m_buildStep->command().toString());
     m_ui->argumentsTextEdit->setPlainText(Utils::QtcProcess::joinArgs(
                                               m_buildStep->arguments()));
     m_ui->resetDefaultsButton->setEnabled(!m_buildStep->isDefault());
@@ -328,9 +299,6 @@ void IosDsymBuildStepConfigWidget::resetDefaults()
 IosDsymBuildStepFactory::IosDsymBuildStepFactory()
 {
     registerStep<IosDsymBuildStep>(Constants::IOS_DSYM_BUILD_STEP_ID);
-    setSupportedStepLists({ProjectExplorer::Constants::BUILDSTEPS_CLEAN,
-                           ProjectExplorer::Constants::BUILDSTEPS_BUILD,
-                           ProjectExplorer::Constants::BUILDSTEPS_DEPLOY});
     setSupportedDeviceTypes({Constants::IOS_DEVICE_TYPE,
                              Constants::IOS_SIMULATOR_TYPE});
     setDisplayName("dsymutil");
