@@ -28,19 +28,27 @@
 #include "nimconstants.h"
 #include "editor/nimeditorfactory.h"
 #include "editor/nimhighlighter.h"
+#include "project/nimblerunconfiguration.h"
+#include "project/nimblebuildconfiguration.h"
 #include "project/nimbuildconfiguration.h"
 #include "project/nimcompilerbuildstep.h"
 #include "project/nimcompilercleanstep.h"
 #include "project/nimproject.h"
+#include "project/nimbleproject.h"
 #include "project/nimrunconfiguration.h"
 #include "project/nimtoolchainfactory.h"
+#include "project/nimblebuildstep.h"
+#include "project/nimbletaskstep.h"
 #include "settings/nimcodestylepreferencesfactory.h"
 #include "settings/nimcodestylesettingspage.h"
+#include "settings/nimtoolssettingspage.h"
 #include "settings/nimsettings.h"
+#include "suggest/nimsuggestcache.h"
 
 #include <coreplugin/fileiconprovider.h>
 #include <projectexplorer/projectmanager.h>
 #include <projectexplorer/toolchainmanager.h>
+#include <projectexplorer/runcontrol.h>
 #include <texteditor/snippets/snippetprovider.h>
 
 using namespace Utils;
@@ -51,13 +59,43 @@ namespace Nim {
 class NimPluginPrivate
 {
 public:
+    NimPluginPrivate()
+        : toolsSettingsPage(&settings)
+    {
+        Suggest::NimSuggestCache::instance().setExecutablePath(settings.nimSuggestPath());
+        QObject::connect(&settings, &NimSettings::nimSuggestPathChanged,
+                         &Suggest::NimSuggestCache::instance(),
+                         &Suggest::NimSuggestCache::setExecutablePath);
+    }
+
     NimSettings settings;
     NimEditorFactory editorFactory;
     NimBuildConfigurationFactory buildConfigFactory;
-    NimRunConfigurationFactory runConfigFactory;
+    NimbleBuildConfigurationFactory nimbleBuildConfigFactory;
+    NimRunConfigurationFactory nimRunConfigFactory;
+    NimbleRunConfigurationFactory nimbleRunConfigFactory;
+    NimbleTestConfigurationFactory nimbleTestConfigFactory;
+    RunWorkerFactory nimRunWorkerFactory {
+        RunWorkerFactory::make<SimpleTargetRunner>(),
+        {ProjectExplorer::Constants::NORMAL_RUN_MODE},
+        {nimRunConfigFactory.id()}
+    };
+    RunWorkerFactory nimbleRunWorkerFactory {
+        RunWorkerFactory::make<SimpleTargetRunner>(),
+        {ProjectExplorer::Constants::NORMAL_RUN_MODE},
+        {nimbleRunConfigFactory.id()}
+    };
+    RunWorkerFactory nimbleTestWorkerFactory {
+        RunWorkerFactory::make<SimpleTargetRunner>(),
+        {ProjectExplorer::Constants::NORMAL_RUN_MODE},
+        {nimbleTestConfigFactory.id()}
+    };
+    NimbleBuildStepFactory nimbleBuildStepFactory;
+    NimbleTaskStepFactory nimbleTaskStepFactory;
     NimCompilerBuildStepFactory buildStepFactory;
     NimCompilerCleanStepFactory cleanStepFactory;
     NimCodeStyleSettingsPage codeStyleSettingsPage;
+    NimToolsSettingsPage toolsSettingsPage;
     NimCodeStylePreferencesFactory codeStylePreferencesPage;
     NimToolChainFactory toolChainFactory;
 };
@@ -76,14 +114,12 @@ bool NimPlugin::initialize(const QStringList &arguments, QString *errorMessage)
 
     ToolChainManager::registerLanguage(Constants::C_NIMLANGUAGE_ID, Constants::C_NIMLANGUAGE_NAME);
 
-    RunControl::registerWorker<NimRunConfiguration, SimpleTargetRunner>
-            (ProjectExplorer::Constants::NORMAL_RUN_MODE);
-
     TextEditor::SnippetProvider::registerGroup(Constants::C_NIMSNIPPETSGROUP_ID,
                                                tr("Nim", "SnippetProvider"),
                                                &NimEditorFactory::decorateEditor);
 
     ProjectManager::registerProjectType<NimProject>(Constants::C_NIM_PROJECT_MIMETYPE);
+    ProjectManager::registerProjectType<NimbleProject>(Constants::C_NIMBLE_MIMETYPE);
 
     return true;
 }
@@ -92,10 +128,12 @@ void NimPlugin::extensionsInitialized()
 {
     // Add MIME overlay icons (these icons displayed at Project dock panel)
     const QIcon icon = Utils::Icon({{":/nim/images/settingscategory_nim.png",
-                                     Utils::Theme::PanelTextColorDark}}, Utils::Icon::Tint).icon();
+            Utils::Theme::PanelTextColorDark
+        }}, Utils::Icon::Tint).icon();
     if (!icon.isNull()) {
         Core::FileIconProvider::registerIconOverlayForMimeType(icon, Constants::C_NIM_MIMETYPE);
         Core::FileIconProvider::registerIconOverlayForMimeType(icon, Constants::C_NIM_SCRIPT_MIMETYPE);
+        Core::FileIconProvider::registerIconOverlayForMimeType(icon, Constants::C_NIMBLE_MIMETYPE);
     }
 }
 
