@@ -32,18 +32,22 @@
 #include "runconfiguration.h"
 #include "target.h"
 
+#include <coreplugin/variablechooser.h>
 #include <utils/utilsicons.h>
 #include <utils/fancylineedit.h>
 #include <utils/pathchooser.h>
 #include <utils/qtcprocess.h>
 
 #include <QCheckBox>
+#include <QComboBox>
 #include <QLabel>
 #include <QLineEdit>
 #include <QFormLayout>
 #include <QSpinBox>
 #include <QToolButton>
 #include <QTextEdit>
+#include <QRadioButton>
+#include <QButtonGroup>
 
 using namespace Utils;
 
@@ -53,6 +57,7 @@ namespace Internal {
 class BaseBoolAspectPrivate
 {
 public:
+    BaseBoolAspect::LabelPlacement m_labelPlacement = BaseBoolAspect::LabelPlacement::AtCheckBox;
     bool m_value = false;
     bool m_defaultValue = false;
     QString m_label;
@@ -60,10 +65,31 @@ public:
     QPointer<QCheckBox> m_checkBox; // Owned by configuration widget
 };
 
+class BaseSelectionAspectPrivate
+{
+public:
+    int m_value = 0;
+    int m_defaultValue = 0;
+    BaseSelectionAspect::DisplayStyle m_displayStyle
+            = BaseSelectionAspect::DisplayStyle::RadioButtons;
+    struct Option { QString displayName; QString tooltip; };
+    QVector<Option> m_options;
+
+    // These are all owned by the configuration widget.
+    QList<QPointer<QRadioButton>> m_buttons;
+    QPointer<QComboBox> m_comboBox;
+    QPointer<QLabel> m_label;
+    QPointer<QButtonGroup> m_buttonGroup;
+};
+
 class BaseStringAspectPrivate
 {
 public:
     BaseStringAspect::DisplayStyle m_displayStyle = BaseStringAspect::LabelDisplay;
+    BaseStringAspect::CheckBoxPlacement m_checkBoxPlacement
+        = BaseStringAspect::CheckBoxPlacement::Right;
+    BaseStringAspect::UncheckedSemantics m_uncheckedSemantics
+        = BaseStringAspect::UncheckedSemantics::Disabled;
     QString m_labelText;
     std::function<QString(const QString &)> m_displayFilter;
     std::unique_ptr<BaseBoolAspect> m_checker;
@@ -78,8 +104,20 @@ public:
     QPointer<FancyLineEdit> m_lineEditDisplay;
     QPointer<PathChooser> m_pathChooserDisplay;
     QPointer<QTextEdit> m_textEditDisplay;
+    Utils::MacroExpanderProvider m_expanderProvider;
     QPixmap m_labelPixmap;
     Utils::FilePath m_baseFileName;
+    bool m_readOnly = false;
+    bool m_showToolTipOnLabel = false;
+
+    template<class Widget> void updateWidgetFromCheckStatus(Widget *w)
+    {
+        const bool enabled = !m_checker || m_checker->value();
+        if (m_uncheckedSemantics == BaseStringAspect::UncheckedSemantics::Disabled)
+            w->setEnabled(enabled);
+        else
+            w->setReadOnly(!enabled);
+    }
 };
 
 class BaseIntegerAspectPrivate
@@ -138,12 +176,12 @@ void BaseStringAspect::toMap(QVariantMap &map) const
         d->m_checker->toMap(map);
 }
 
-FilePath BaseStringAspect::fileName() const
+FilePath BaseStringAspect::filePath() const
 {
     return FilePath::fromString(d->m_value);
 }
 
-void BaseStringAspect::setFileName(const FilePath &val)
+void BaseStringAspect::setFilePath(const FilePath &val)
 {
     setValue(val.toString());
 }
@@ -162,6 +200,12 @@ void BaseStringAspect::setLabelPixmap(const QPixmap &labelPixmap)
         d->m_label->setPixmap(labelPixmap);
 }
 
+void BaseStringAspect::setShowToolTipOnLabel(bool show)
+{
+    d->m_showToolTipOnLabel = show;
+    update();
+}
+
 QString BaseStringAspect::labelText() const
 {
     return d->m_labelText;
@@ -177,6 +221,12 @@ bool BaseStringAspect::isChecked() const
     return !d->m_checker || d->m_checker->value();
 }
 
+void BaseStringAspect::setChecked(bool checked)
+{
+    QTC_ASSERT(d->m_checker, return);
+    d->m_checker->setValue(checked);
+}
+
 void BaseStringAspect::setDisplayStyle(DisplayStyle displayStyle)
 {
     d->m_displayStyle = displayStyle;
@@ -218,44 +268,84 @@ void BaseStringAspect::setBaseFileName(const FilePath &baseFileName)
 {
     d->m_baseFileName = baseFileName;
     if (d->m_pathChooserDisplay)
-        d->m_pathChooserDisplay->setBaseFileName(baseFileName);
+        d->m_pathChooserDisplay->setBaseDirectory(baseFileName);
 }
 
-void BaseStringAspect::addToConfigurationLayout(QFormLayout *layout)
+void BaseStringAspect::setReadOnly(bool readOnly)
+{
+    d->m_readOnly = readOnly;
+    if (d->m_pathChooserDisplay)
+        d->m_pathChooserDisplay->setReadOnly(readOnly);
+    if (d->m_lineEditDisplay)
+        d->m_lineEditDisplay->setReadOnly(readOnly);
+    if (d->m_textEditDisplay)
+        d->m_textEditDisplay->setReadOnly(readOnly);
+}
+
+void BaseStringAspect::setMacroExpanderProvider(const MacroExpanderProvider &expanderProvider)
+{
+    d->m_expanderProvider = expanderProvider;
+}
+
+void BaseStringAspect::setUncheckedSemantics(BaseStringAspect::UncheckedSemantics semantics)
+{
+    d->m_uncheckedSemantics = semantics;
+}
+
+void BaseStringAspect::addToLayout(LayoutBuilder &builder)
 {
     QTC_CHECK(!d->m_label);
-    QWidget *parent = layout->parentWidget();
-    d->m_label = new QLabel(parent);
+
+    if (d->m_checker && d->m_checkBoxPlacement == CheckBoxPlacement::Top) {
+        d->m_checker->addToLayout(builder);
+        builder.startNewRow();
+    }
+
+    d->m_label = new QLabel;
     d->m_label->setTextInteractionFlags(Qt::TextSelectableByMouse);
     d->m_label->setText(d->m_labelText);
     if (!d->m_labelPixmap.isNull())
         d->m_label->setPixmap(d->m_labelPixmap);
+    builder.addItem(d->m_label.data());
+
+    const auto useMacroExpander = [this, &builder](QWidget *w) {
+        if (!d->m_expanderProvider)
+            return;
+        const auto chooser = new Core::VariableChooser(builder.layout()->parentWidget());
+        chooser->addSupportedWidget(w);
+        chooser->addMacroExpanderProvider(d->m_expanderProvider);
+    };
 
-    auto hbox = new QHBoxLayout;
     switch (d->m_displayStyle) {
     case PathChooserDisplay:
-        d->m_pathChooserDisplay = new PathChooser(parent);
+        d->m_pathChooserDisplay = new PathChooser;
         d->m_pathChooserDisplay->setExpectedKind(d->m_expectedKind);
         if (!d->m_historyCompleterKey.isEmpty())
             d->m_pathChooserDisplay->setHistoryCompleter(d->m_historyCompleterKey);
         d->m_pathChooserDisplay->setEnvironment(d->m_environment);
-        d->m_pathChooserDisplay->setBaseFileName(d->m_baseFileName);
+        d->m_pathChooserDisplay->setBaseDirectory(d->m_baseFileName);
+        d->m_pathChooserDisplay->setReadOnly(d->m_readOnly);
+        useMacroExpander(d->m_pathChooserDisplay->lineEdit());
         connect(d->m_pathChooserDisplay, &PathChooser::pathChanged,
                 this, &BaseStringAspect::setValue);
-        hbox->addWidget(d->m_pathChooserDisplay);
+        builder.addItem(d->m_pathChooserDisplay.data());
         break;
     case LineEditDisplay:
-        d->m_lineEditDisplay = new FancyLineEdit(parent);
+        d->m_lineEditDisplay = new FancyLineEdit;
         d->m_lineEditDisplay->setPlaceholderText(d->m_placeHolderText);
         if (!d->m_historyCompleterKey.isEmpty())
             d->m_lineEditDisplay->setHistoryCompleter(d->m_historyCompleterKey);
+        d->m_lineEditDisplay->setReadOnly(d->m_readOnly);
+        useMacroExpander(d->m_lineEditDisplay);
         connect(d->m_lineEditDisplay, &FancyLineEdit::textEdited,
                 this, &BaseStringAspect::setValue);
-        hbox->addWidget(d->m_lineEditDisplay);
+        builder.addItem(d->m_lineEditDisplay.data());
         break;
     case TextEditDisplay:
-        d->m_textEditDisplay = new QTextEdit(parent);
+        d->m_textEditDisplay = new QTextEdit;
         d->m_textEditDisplay->setPlaceholderText(d->m_placeHolderText);
+        d->m_textEditDisplay->setReadOnly(d->m_readOnly);
+        useMacroExpander(d->m_textEditDisplay);
         connect(d->m_textEditDisplay, &QTextEdit::textChanged, this, [this] {
             const QString value = d->m_textEditDisplay->document()->toPlainText();
             if (value != d->m_value) {
@@ -263,23 +353,17 @@ void BaseStringAspect::addToConfigurationLayout(QFormLayout *layout)
                 emit changed();
             }
         });
-        hbox->addWidget(d->m_textEditDisplay);
+        builder.addItem(d->m_textEditDisplay.data());
         break;
     case LabelDisplay:
-        d->m_labelDisplay = new QLabel(parent);
+        d->m_labelDisplay = new QLabel;
         d->m_labelDisplay->setTextInteractionFlags(Qt::TextSelectableByMouse);
-        hbox->addWidget(d->m_labelDisplay);
+        builder.addItem(d->m_labelDisplay.data());
         break;
     }
 
-    if (d->m_checker) {
-        auto form = new QFormLayout;
-        form->setContentsMargins(0, 0, 0, 0);
-        form->setFormAlignment(Qt::AlignLeft | Qt::AlignVCenter);
-        d->m_checker->addToConfigurationLayout(form);
-        hbox->addLayout(form);
-    }
-    layout->addRow(d->m_label, hbox);
+    if (d->m_checker && d->m_checkBoxPlacement == CheckBoxPlacement::Right)
+        d->m_checker->addToLayout(builder);
 
     update();
 }
@@ -289,25 +373,25 @@ void BaseStringAspect::update()
     const QString displayedString = d->m_displayFilter ? d->m_displayFilter(d->m_value)
                                                        : d->m_value;
 
-    const bool enabled = !d->m_checker || d->m_checker->value();
-
     if (d->m_pathChooserDisplay) {
         d->m_pathChooserDisplay->setFileName(FilePath::fromString(displayedString));
-        d->m_pathChooserDisplay->setEnabled(enabled);
+        d->updateWidgetFromCheckStatus(d->m_pathChooserDisplay.data());
     }
 
     if (d->m_lineEditDisplay) {
         d->m_lineEditDisplay->setTextKeepingActiveCursor(displayedString);
-        d->m_lineEditDisplay->setEnabled(enabled);
+        d->updateWidgetFromCheckStatus(d->m_lineEditDisplay.data());
     }
 
     if (d->m_textEditDisplay) {
         d->m_textEditDisplay->setText(displayedString);
-        d->m_textEditDisplay->setEnabled(enabled);
+        d->updateWidgetFromCheckStatus(d->m_textEditDisplay.data());
     }
 
-    if (d->m_labelDisplay)
+    if (d->m_labelDisplay) {
         d->m_labelDisplay->setText(displayedString);
+        d->m_labelDisplay->setToolTip(d->m_showToolTipOnLabel ? displayedString : QString());
+    }
 
     if (d->m_label) {
         d->m_label->setText(d->m_labelText);
@@ -316,15 +400,20 @@ void BaseStringAspect::update()
     }
 }
 
-void BaseStringAspect::makeCheckable(const QString &checkerLabel, const QString &checkerKey)
+void BaseStringAspect::makeCheckable(CheckBoxPlacement checkBoxPlacement,
+                                     const QString &checkerLabel, const QString &checkerKey)
 {
     QTC_ASSERT(!d->m_checker, return);
+    d->m_checkBoxPlacement = checkBoxPlacement;
     d->m_checker.reset(new BaseBoolAspect);
-    d->m_checker->setLabel(checkerLabel);
+    d->m_checker->setLabel(checkerLabel, checkBoxPlacement == CheckBoxPlacement::Top
+                           ? BaseBoolAspect::LabelPlacement::InExtraLabel
+                           : BaseBoolAspect::LabelPlacement::AtCheckBox);
     d->m_checker->setSettingsKey(checkerKey);
 
     connect(d->m_checker.get(), &BaseBoolAspect::changed, this, &BaseStringAspect::update);
     connect(d->m_checker.get(), &BaseBoolAspect::changed, this, &BaseStringAspect::changed);
+    connect(d->m_checker.get(), &BaseBoolAspect::changed, this, &BaseStringAspect::checkedChanged);
 
     update();
 }
@@ -341,13 +430,19 @@ BaseBoolAspect::BaseBoolAspect(const QString &settingsKey)
 
 BaseBoolAspect::~BaseBoolAspect() = default;
 
-void BaseBoolAspect::addToConfigurationLayout(QFormLayout *layout)
+void BaseBoolAspect::addToLayout(LayoutBuilder &builder)
 {
     QTC_CHECK(!d->m_checkBox);
-    d->m_checkBox = new QCheckBox(d->m_label, layout->parentWidget());
+    d->m_checkBox = new QCheckBox();
+    if (d->m_labelPlacement == LabelPlacement::AtCheckBox) {
+        d->m_checkBox->setText(d->m_label);
+        builder.addItem(new QLabel);
+    } else {
+        builder.addItem(d->m_label);
+    }
     d->m_checkBox->setChecked(d->m_value);
     d->m_checkBox->setToolTip(d->m_tooltip);
-    layout->addRow(QString(), d->m_checkBox);
+    builder.addItem(d->m_checkBox.data());
     connect(d->m_checkBox.data(), &QAbstractButton::clicked, this, [this] {
         d->m_value = d->m_checkBox->isChecked();
         emit changed();
@@ -387,9 +482,10 @@ void BaseBoolAspect::setValue(bool value)
         d->m_checkBox->setChecked(d->m_value);
 }
 
-void BaseBoolAspect::setLabel(const QString &label)
+void BaseBoolAspect::setLabel(const QString &label, LabelPlacement labelPlacement)
 {
     d->m_label = label;
+    d->m_labelPlacement = labelPlacement;
 }
 
 void BaseBoolAspect::setToolTip(const QString &tooltip)
@@ -397,6 +493,108 @@ void BaseBoolAspect::setToolTip(const QString &tooltip)
     d->m_tooltip = tooltip;
 }
 
+/*!
+    \class ProjectExplorer::BaseSelectionAspect
+*/
+
+BaseSelectionAspect::BaseSelectionAspect()
+    : d(new Internal::BaseSelectionAspectPrivate)
+{}
+
+BaseSelectionAspect::~BaseSelectionAspect() = default;
+
+void BaseSelectionAspect::addToLayout(LayoutBuilder &builder)
+{
+    QTC_CHECK(d->m_buttonGroup == nullptr);
+    QTC_CHECK(!d->m_comboBox);
+    QTC_ASSERT(d->m_buttons.isEmpty(), d->m_buttons.clear());
+
+    switch (d->m_displayStyle) {
+    case DisplayStyle::RadioButtons:
+        d->m_buttonGroup = new QButtonGroup;
+        d->m_buttonGroup->setExclusive(true);
+        for (int i = 0, n = d->m_options.size(); i < n; ++i) {
+            const Internal::BaseSelectionAspectPrivate::Option &option = d->m_options.at(i);
+            auto button = new QRadioButton(option.displayName);
+            button->setChecked(i == d->m_value);
+            button->setToolTip(option.tooltip);
+            builder.addItems(QString(), button);
+            d->m_buttons.append(button);
+            d->m_buttonGroup->addButton(button);
+            connect(button, &QAbstractButton::clicked, this, [this, i] {
+                d->m_value = i;
+                emit changed();
+            });
+        }
+        break;
+    case DisplayStyle::ComboBox:
+        d->m_label = new QLabel(displayName());
+        d->m_comboBox = new QComboBox;
+        for (int i = 0, n = d->m_options.size(); i < n; ++i)
+            d->m_comboBox->addItem(d->m_options.at(i).displayName);
+        connect(d->m_comboBox.data(), QOverload<int>::of(&QComboBox::activated), this,
+                [this](int index) { d->m_value = index; emit changed(); });
+        d->m_comboBox->setCurrentIndex(d->m_value);
+        builder.addItems(d->m_label.data(), d->m_comboBox.data());
+        break;
+    }
+}
+
+void BaseSelectionAspect::fromMap(const QVariantMap &map)
+{
+    d->m_value = map.value(settingsKey(), d->m_defaultValue).toInt();
+}
+
+void BaseSelectionAspect::toMap(QVariantMap &data) const
+{
+    data.insert(settingsKey(), d->m_value);
+}
+
+void BaseSelectionAspect::setVisibleDynamic(bool visible)
+{
+    if (d->m_label)
+        d->m_label->setVisible(visible);
+    if (d->m_comboBox)
+        d->m_comboBox->setVisible(visible);
+    for (QRadioButton * const button : qAsConst(d->m_buttons))
+        button->setVisible(visible);
+}
+
+int BaseSelectionAspect::defaultValue() const
+{
+    return d->m_defaultValue;
+}
+
+void BaseSelectionAspect::setDefaultValue(int defaultValue)
+{
+    d->m_defaultValue = defaultValue;
+}
+
+void BaseSelectionAspect::setDisplayStyle(BaseSelectionAspect::DisplayStyle style)
+{
+    d->m_displayStyle = style;
+}
+
+int BaseSelectionAspect::value() const
+{
+    return d->m_value;
+}
+
+void BaseSelectionAspect::setValue(int value)
+{
+    d->m_value = value;
+    if (d->m_buttonGroup && 0 <= value && value < d->m_buttons.size())
+        d->m_buttons.at(value)->setChecked(true);
+    else if (d->m_comboBox) {
+        d->m_comboBox->setCurrentIndex(value);
+    }
+}
+
+void BaseSelectionAspect::addOption(const QString &displayName, const QString &toolTip)
+{
+    d->m_options.append({displayName, toolTip});
+}
+
 /*!
     \class ProjectExplorer::BaseIntegerAspect
 */
@@ -409,10 +607,10 @@ BaseIntegerAspect::BaseIntegerAspect()
 
 BaseIntegerAspect::~BaseIntegerAspect() = default;
 
-void BaseIntegerAspect::addToConfigurationLayout(QFormLayout *layout)
+void BaseIntegerAspect::addToLayout(LayoutBuilder &builder)
 {
     QTC_CHECK(!d->m_spinBox);
-    d->m_spinBox = new QSpinBox(layout->parentWidget());
+    d->m_spinBox = new QSpinBox;
     d->m_spinBox->setValue(int(d->m_value / d->m_displayScaleFactor));
     d->m_spinBox->setDisplayIntegerBase(d->m_displayIntegerBase);
     d->m_spinBox->setPrefix(d->m_prefix);
@@ -420,7 +618,7 @@ void BaseIntegerAspect::addToConfigurationLayout(QFormLayout *layout)
     if (d->m_maximumValue.isValid() && d->m_maximumValue.isValid())
         d->m_spinBox->setRange(int(d->m_minimumValue.toLongLong() / d->m_displayScaleFactor),
                                int(d->m_maximumValue.toLongLong() / d->m_displayScaleFactor));
-    layout->addRow(d->m_label, d->m_spinBox);
+    builder.addItems(d->m_label, d->m_spinBox.data());
     connect(d->m_spinBox.data(), QOverload<int>::of(&QSpinBox::valueChanged),
             this, [this](int value) {
         d->m_value = value * d->m_displayScaleFactor;
@@ -481,4 +679,34 @@ void BaseIntegerAspect::setDisplayScaleFactor(qint64 factor)
     d->m_displayScaleFactor = factor;
 }
 
+BaseTriStateAspect::BaseTriStateAspect()
+{
+    setDisplayStyle(DisplayStyle::ComboBox);
+    setDefaultValue(2);
+    addOption(tr("Enable"));
+    addOption(tr("Disable"));
+    addOption(tr("Leave at Default"));
+}
+
+TriState BaseTriStateAspect::setting() const
+{
+    return TriState::fromVariant(value());
+}
+
+void BaseTriStateAspect::setSetting(TriState setting)
+{
+    setValue(setting.toVariant().toInt());
+}
+
+const TriState TriState::Enabled{TriState::EnabledValue};
+const TriState TriState::Disabled{TriState::DisabledValue};
+const TriState TriState::Default{TriState::DefaultValue};
+
+TriState TriState::fromVariant(const QVariant &variant)
+{
+    int v = variant.toInt();
+    QTC_ASSERT(v == EnabledValue || v == DisabledValue || v == DefaultValue, v = DefaultValue);
+    return TriState(Value(v));
+}
+
 } // namespace ProjectExplorer
