@@ -42,15 +42,9 @@ namespace Utils { class MimeType; }
 
 namespace ProjectExplorer {
 
+class BuildSystem;
 class Project;
 
-enum class NodeType : quint16 {
-    File = 1,
-    Folder,
-    VirtualFolder,
-    Project
-};
-
 // File types common for qt projects
 enum class FileType : quint16 {
     Unknown = 0,
@@ -64,10 +58,13 @@ enum class FileType : quint16 {
     FileTypeSize
 };
 
+enum class ProductType { App, Lib, Other, None };
+
 enum ProjectAction {
     // Special value to indicate that the actions are handled by the parent
     InheritedFromParent,
     AddSubProject,
+    AddExistingProject,
     RemoveSubProject,
     // Let's the user select to which project file
     // the file is added
@@ -84,14 +81,14 @@ enum ProjectAction {
     // DeleteFile is a define on windows...
     EraseFile,
     Rename,
-    DuplicateFile,
     // hides actions that use the path(): Open containing folder, open terminal here and Find in Directory
     HidePathActions,
     HideFileActions,
     HideFolderActions,
-    HasSubProjectRunConfigurations
 };
 
+enum class RemovedFilesFromProject { Ok, Wildcard, Error };
+
 class FileNode;
 class FolderNode;
 class ProjectNode;
@@ -111,8 +108,11 @@ public:
     };
 
     virtual ~Node();
-    Node(const Node &other) = delete;
-    NodeType nodeType() const;
+
+    virtual bool isFolderNodeType() const { return false; }
+    virtual bool isProjectNodeType() const { return false; }
+    virtual bool isVirtualFolderType() const { return false; }
+
     int priority() const;
 
     ProjectNode *parentProjectNode() const; // parent project, will be nullptr for the top-level project
@@ -125,9 +125,10 @@ public:
                                      // or node->parentProjectNode() for all other cases.
     const ProjectNode *managingProject() const; // see above.
 
-    const Utils::FileName &filePath() const;  // file system path
+    Project *getProject() const;
+
+    const Utils::FilePath &filePath() const;  // file system path
     int line() const;
-    QByteArray id() const;
     virtual QString displayName() const;
     virtual QString tooltip() const;
     bool isEnabled() const;
@@ -137,7 +138,7 @@ public:
     virtual bool supportsAction(ProjectAction action, const Node *node) const;
 
     void setEnabled(bool enabled);
-    void setAbsoluteFilePathAndLine(const Utils::FileName &filePath, int line);
+    void setAbsoluteFilePathAndLine(const Utils::FilePath &filePath, int line);
 
     virtual FileNode *asFileNode() { return nullptr; }
     virtual const FileNode *asFileNode() const { return nullptr; }
@@ -154,24 +155,31 @@ public:
     void setParentFolderNode(FolderNode *parentFolder);
 
     void setListInProject(bool l);
+    void setIsGenerated(bool g);
+    void setPriority(int priority);
+    void setLine(int line);
 
     static FileType fileTypeForMimeType(const Utils::MimeType &mt);
-    static FileType fileTypeForFileName(const Utils::FileName &file);
+    static FileType fileTypeForFileName(const Utils::FilePath &file);
+
+    QString path() const { return pathOrDirectory(false); }
+    QString directory() const { return pathOrDirectory(true); }
 
 protected:
-    Node(NodeType nodeType, const Utils::FileName &filePath, int line = -1,
-         const QByteArray &id = {});
+    Node();
+    Node(const Node &other) = delete;
+    bool operator=(const Node &other) = delete;
 
-    void setPriority(int priority);
-    void setIsGenerated(bool g);
+    void setFilePath(const Utils::FilePath &filePath);
 
 private:
+    QString pathOrDirectory(bool dir) const;
+
     FolderNode *m_parentFolderNode = nullptr;
-    Utils::FileName m_filePath;
-    QByteArray m_nodeId;
+    Utils::FilePath m_filePath;
     int m_line = -1;
     int m_priority = DefaultPriority;
-    const NodeType m_nodeType;
+
     enum NodeFlag : quint16 {
         FlagNone = 0,
         FlagIsEnabled = 1 << 0,
@@ -184,8 +192,7 @@ private:
 class PROJECTEXPLORER_EXPORT FileNode : public Node
 {
 public:
-    FileNode(const Utils::FileName &filePath, const FileType fileType, bool generated, int line = -1,
-             const QByteArray &id = {});
+    FileNode(const Utils::FilePath &filePath, const FileType fileType);
 
     FileNode *clone() const;
 
@@ -195,10 +202,11 @@ public:
     const FileNode *asFileNode() const final { return this; }
 
     static QList<FileNode *>
-    scanForFiles(const Utils::FileName &directory,
-                 const std::function<FileNode *(const Utils::FileName &fileName)> factory,
+    scanForFiles(const Utils::FilePath &directory,
+                 const std::function<FileNode *(const Utils::FilePath &fileName)> factory,
                  QFutureInterface<QList<FileNode *>> *future = nullptr);
     bool supportsAction(ProjectAction action, const Node *node) const override;
+    QString displayName() const override;
 
 private:
     FileType m_fileType;
@@ -208,12 +216,13 @@ private:
 class PROJECTEXPLORER_EXPORT FolderNode : public Node
 {
 public:
-    explicit FolderNode(const Utils::FileName &folderPath, NodeType nodeType = NodeType::Folder,
-                        const QString &displayName = QString(), const QByteArray &id = {});
+    explicit FolderNode(const Utils::FilePath &folderPath);
 
     QString displayName() const override;
     QIcon icon() const;
 
+    bool isFolderNodeType() const override { return true; }
+
     Node *findNode(const std::function<bool(Node *)> &filter);
     QList<Node *> findNodes(const std::function<bool(Node *)> &filter);
 
@@ -225,20 +234,20 @@ public:
     ProjectNode *findProjectNode(const std::function<bool(const ProjectNode *)> &predicate);
     const QList<Node *> nodes() const;
     QList<FileNode *> fileNodes() const;
-    FileNode *fileNode(const Utils::FileName &file) const;
+    FileNode *fileNode(const Utils::FilePath &file) const;
     QList<FolderNode *> folderNodes() const;
-    using FolderNodeFactory = std::function<std::unique_ptr<FolderNode>(const Utils::FileName &)>;
-    void addNestedNodes(std::vector<std::unique_ptr<FileNode>> &&files,
-                        const Utils::FileName &overrideBaseDir = Utils::FileName(),
-                        const FolderNodeFactory &factory
-                        = [](const Utils::FileName &fn) { return std::make_unique<FolderNode>(fn); });
-    void addNestedNode(std::unique_ptr<FileNode> &&fileNode,
-                       const Utils::FileName &overrideBaseDir = Utils::FileName(),
-                       const FolderNodeFactory &factory
-                       = [](const Utils::FileName &fn) { return std::make_unique<FolderNode>(fn); });
-    void compress();
+    FolderNode *folderNode(const Utils::FilePath &directory) const;
 
-    bool isAncesterOf(Node *n);
+    using FolderNodeFactory = std::function<std::unique_ptr<FolderNode>(const Utils::FilePath &)>;
+    void addNestedNodes(std::vector<std::unique_ptr<FileNode>> &&files,
+                        const Utils::FilePath &overrideBaseDir = Utils::FilePath(),
+                        const FolderNodeFactory &factory
+                        = [](const Utils::FilePath &fn) { return std::make_unique<FolderNode>(fn); });
+    void addNestedNode(std::unique_ptr<FileNode> &&fileNode,
+                       const Utils::FilePath &overrideBaseDir = Utils::FilePath(),
+                       const FolderNodeFactory &factory
+                       = [](const Utils::FilePath &fn) { return std::make_unique<FolderNode>(fn); });
+    void compress();
 
     // takes ownership of newNode.
     // Will delete newNode if oldNode is not a child of this node.
@@ -247,27 +256,40 @@ public:
     void setDisplayName(const QString &name);
     void setIcon(const QIcon &icon);
 
-    class LocationInfo {
+    class LocationInfo
+    {
     public:
-        LocationInfo(const QString &dn, const Utils::FileName &p, const int l = -1) :
-            path(p), line(l), displayName(dn) { }
+        LocationInfo() = default;
+        LocationInfo(const QString &dn,
+                     const Utils::FilePath &p,
+                     const int l = 0,
+                     const unsigned int prio = 0)
+            : path(p)
+            , line(l)
+            , priority(prio)
+            , displayName(dn)
+        {}
 
-        Utils::FileName path;
+        Utils::FilePath path;
         int line = -1;
+        unsigned int priority = 0;
         QString displayName;
     };
-    void setLocationInfo(const QList<LocationInfo> &info);
-    const QList<LocationInfo> locationInfo() const;
+    void setLocationInfo(const QVector<LocationInfo> &info);
+    const QVector<LocationInfo> locationInfo() const;
 
-    virtual QString addFileFilter() const;
+    QString addFileFilter() const;
+    void setAddFileFilter(const QString &filter) { m_addFileFilter = filter; }
 
     bool supportsAction(ProjectAction action, const Node *node) const override;
 
     virtual bool addFiles(const QStringList &filePaths, QStringList *notAdded = nullptr);
-    virtual bool removeFiles(const QStringList &filePaths, QStringList *notRemoved = nullptr);
+    virtual RemovedFilesFromProject removeFiles(const QStringList &filePaths,
+                                                QStringList *notRemoved = nullptr);
     virtual bool deleteFiles(const QStringList &filePaths);
     virtual bool canRenameFile(const QString &filePath, const QString &newFilePath);
     virtual bool renameFile(const QString &filePath, const QString &newFilePath);
+    virtual bool addDependencies(const QStringList &dependencies);
 
     class AddNewInformation
     {
@@ -284,11 +306,12 @@ public:
 
     // determines if node will be shown in the flat view, by default folder and projects aren't shown
     virtual bool showInSimpleTree() const;
+
     // determines if node will always be shown when hiding empty directories
-    virtual bool showWhenEmpty() const;
+    bool showWhenEmpty() const;
+    void setShowWhenEmpty(bool showWhenEmpty);
 
     void addNode(std::unique_ptr<Node> &&node);
-    std::unique_ptr<Node> takeNode(Node *node);
 
     bool isEmpty() const;
 
@@ -299,48 +322,57 @@ protected:
     virtual void handleSubTreeChanged(FolderNode *node);
 
     std::vector<std::unique_ptr<Node>> m_nodes;
-    QList<LocationInfo> m_locations;
+    QVector<LocationInfo> m_locations;
 
 private:
+    std::unique_ptr<Node> takeNode(Node *node);
+
     QString m_displayName;
+    QString m_addFileFilter;
     mutable QIcon m_icon;
+    bool m_showWhenEmpty = false;
 };
 
 class PROJECTEXPLORER_EXPORT VirtualFolderNode : public FolderNode
 {
 public:
-    explicit VirtualFolderNode(const Utils::FileName &folderPath, int priority,
-                               const QByteArray &id = {});
+    explicit VirtualFolderNode(const Utils::FilePath &folderPath);
 
-    void setAddFileFilter(const QString &filter) { m_addFileFilter = filter; }
-    QString addFileFilter() const override;
-
-private:
-    QString m_addFileFilter;
+    bool isFolderNodeType() const override { return false; }
+    bool isVirtualFolderType() const override { return true; }
 };
 
 // Documentation inside.
 class PROJECTEXPLORER_EXPORT ProjectNode : public FolderNode
 {
 public:
+    explicit ProjectNode(const Utils::FilePath &projectFilePath);
+
     virtual bool canAddSubProject(const QString &proFilePath) const;
     virtual bool addSubProject(const QString &proFile);
+    virtual QStringList subProjectFileNamePatterns() const;
     virtual bool removeSubProject(const QString &proFilePath);
-    virtual Utils::optional<Utils::FileName> visibleAfterAddFileAction() const {
+    virtual Utils::optional<Utils::FilePath> visibleAfterAddFileAction() const {
         return Utils::nullopt;
     }
 
-    bool addFiles(const QStringList &filePaths, QStringList *notAdded = nullptr) override;
-    bool removeFiles(const QStringList &filePaths, QStringList *notRemoved = nullptr) override;
-    bool deleteFiles(const QStringList &filePaths) override;
-    bool canRenameFile(const QString &filePath, const QString &newFilePath) override;
-    bool renameFile(const QString &filePath, const QString &newFilePath) override;
-    bool supportsAction(ProjectAction action, const Node *node) const override;
+    bool isFolderNodeType() const override { return false; }
+    bool isProjectNodeType() const override { return true; }
+    bool showInSimpleTree() const override { return true; }
+
+    bool addFiles(const QStringList &filePaths, QStringList *notAdded = nullptr) final;
+    RemovedFilesFromProject removeFiles(const QStringList &filePaths,
+                                        QStringList *notRemoved = nullptr) final;
+    bool deleteFiles(const QStringList &filePaths) final;
+    bool canRenameFile(const QString &filePath, const QString &newFilePath) final;
+    bool renameFile(const QString &filePath, const QString &newFilePath) final;
+    bool addDependencies(const QStringList &dependencies) final;
+    bool supportsAction(ProjectAction action, const Node *node) const final;
 
     // by default returns false
     virtual bool deploysFolder(const QString &folder) const;
 
-    ProjectNode *projectNode(const Utils::FileName &file) const;
+    ProjectNode *projectNode(const Utils::FilePath &file) const;
 
     ProjectNode *asProjectNode() final { return this; }
     const ProjectNode *asProjectNode() const final { return this; }
@@ -352,8 +384,28 @@ public:
     virtual QVariant data(Core::Id role) const;
     virtual bool setData(Core::Id role, const QVariant &value) const;
 
+    bool isProduct() const { return m_productType != ProductType::None; }
+    ProductType productType() const { return m_productType; }
+
+    // TODO: Currently used only for "Build for current run config" functionality, but we should
+    //       probably use it to centralize the node-specific "Build" functionality that
+    //       currently each project manager plugin adds to the context menu by itself.
+    //       The function should then move up to the Node class, so it can also serve the
+    //       "build single file" case.
+    virtual void build() {}
+
+    void setFallbackData(Core::Id key, const QVariant &value);
+
 protected:
-    explicit ProjectNode(const Utils::FileName &projectFilePath, const QByteArray &id = {});
+    void setProductType(ProductType type) { m_productType = type; }
+
+    QString m_target;
+
+private:
+    BuildSystem *buildSystem() const;
+
+    QHash<Core::Id, QVariant> m_fallbackData; // Used in data(), unless overridden.
+    ProductType m_productType = ProductType::None;
 };
 
 class PROJECTEXPLORER_EXPORT ContainerNode : public FolderNode
@@ -364,6 +416,9 @@ public:
     QString displayName() const final;
     bool supportsAction(ProjectAction action, const Node *node) const final;
 
+    bool isFolderNodeType() const override { return false; }
+    bool isProjectNodeType() const override { return true; }
+
     ContainerNode *asContainerNode() final { return this; }
     const ContainerNode *asContainerNode() const final { return this; }
 
