@@ -29,16 +29,10 @@
 
 namespace QmlDesigner {
 
-
-SelectionContext::SelectionContext() :
-    m_toggled(false)
-{
-
-}
+SelectionContext::SelectionContext() = default;
 
 SelectionContext::SelectionContext(AbstractView *view) :
-    m_view(view),
-    m_toggled(false)
+    m_view(view)
 {
 }
 
@@ -52,6 +46,11 @@ ModelNode SelectionContext::targetNode() const
     return m_targetNode;
 }
 
+ModelNode SelectionContext::rootNode() const
+{
+    return view()->rootModelNode();
+}
+
 bool SelectionContext::singleNodeIsSelected() const
 {
     return view()->hasSingleSelectedModelNode();
@@ -122,4 +121,14 @@ bool SelectionContext::isValid() const
     return view() && view()->model();
 }
 
+bool SelectionContext::fastUpdate() const
+{
+    return m_updateMode == UpdateMode::Fast;
+}
+
+void SelectionContext::setUpdateMode(UpdateMode mode)
+{
+    m_updateMode = mode;
+}
+
 } //QmlDesigner
