@@ -158,6 +158,9 @@ public:
     void notifyCurrentStateChanged(const ModelNode &node);
     void notifyCurrentTimelineChanged(const ModelNode &node);
 
+    void notifyRenderImage3DChanged(const QImage &image);
+    void notifyUpdateActiveScene3D(const QVariantMap &sceneState);
+
     void setDocumentMessages(const QList<DocumentMessage> &errors, const QList<DocumentMessage> &warnings);
 
     void notifyRewriterBeginTransaction();
@@ -182,6 +185,8 @@ public:
     void removeImport(const Import &import);
     void changeImports(const QList<Import> &importsToBeAdded, const QList<Import> &importToBeRemoved);
     void notifyImportsChanged(const QList<Import> &addedImports, const QList<Import> &removedImports);
+    void notifyPossibleImportsChanged(const QList<Import> &possibleImports);
+    void notifyUsedImportsChanged(const QList<Import> &usedImportsChanged);
 
 
     //node state property manipulation
