@@ -42,7 +42,6 @@ namespace Internal {
 class AbstractPackagingStepPrivate
 {
 public:
-    BuildConfiguration *currentBuildConfiguration = nullptr;
     QString cachedPackageFilePath;
     QString cachedPackageDirectory;
     bool deploymentDataModified = false;
@@ -54,9 +53,6 @@ AbstractPackagingStep::AbstractPackagingStep(BuildStepList *bsl, Core::Id id)
     : BuildStep(bsl, id)
 {
     d = new Internal::AbstractPackagingStepPrivate;
-    connect(target(), &Target::activeBuildConfigurationChanged,
-            this, &AbstractPackagingStep::handleBuildConfigurationChanged);
-    handleBuildConfigurationChanged();
 
     connect(target(), &Target::deploymentDataChanged,
             this, &AbstractPackagingStep::setDeploymentDataModified);
@@ -71,18 +67,6 @@ AbstractPackagingStep::~AbstractPackagingStep()
     delete d;
 }
 
-void AbstractPackagingStep::handleBuildConfigurationChanged()
-{
-    if (d->currentBuildConfiguration)
-        disconnect(d->currentBuildConfiguration, nullptr, this, nullptr);
-    d->currentBuildConfiguration = buildConfiguration();
-    if (d->currentBuildConfiguration) {
-        connect(d->currentBuildConfiguration, &BuildConfiguration::buildDirectoryChanged,
-                this, &AbstractPackagingStep::packageFilePathChanged);
-    }
-    emit packageFilePathChanged();
-}
-
 QString AbstractPackagingStep::cachedPackageFilePath() const
 {
     return d->cachedPackageFilePath;
@@ -102,8 +86,7 @@ QString AbstractPackagingStep::cachedPackageDirectory() const
 
 QString AbstractPackagingStep::packageDirectory() const
 {
-    return d->currentBuildConfiguration
-            ? d->currentBuildConfiguration->buildDirectory().toString() : QString();
+    return buildConfiguration()->buildDirectory().toString();
 }
 
 bool AbstractPackagingStep::isPackagingNeeded() const
@@ -114,10 +97,8 @@ bool AbstractPackagingStep::isPackagingNeeded() const
 
     const DeploymentData &dd = target()->deploymentData();
     for (int i = 0; i < dd.fileCount(); ++i) {
-        if (Utils::FileUtils::isFileNewerThan(dd.fileAt(i).localFilePath(),
-                packageInfo.lastModified())) {
+        if (dd.fileAt(i).localFilePath().isNewerThan(packageInfo.lastModified()))
             return true;
-        }
     }
 
     return false;
@@ -154,17 +135,13 @@ void AbstractPackagingStep::setDeploymentDataModified()
 
 void AbstractPackagingStep::raiseError(const QString &errorMessage)
 {
-    Task task = Task(Task::Error, errorMessage, Utils::FileName(), -1,
-                     Constants::TASK_CATEGORY_DEPLOYMENT);
-    emit addTask(task);
+    emit addTask(DeploymentTask(Task::Error, errorMessage));
     emit addOutput(errorMessage, BuildStep::OutputFormat::Stderr);
 }
 
 void AbstractPackagingStep::raiseWarning(const QString &warningMessage)
 {
-    Task task = Task(Task::Warning, warningMessage, Utils::FileName(), -1,
-                     Constants::TASK_CATEGORY_DEPLOYMENT);
-    emit addTask(task);
+    emit addTask(DeploymentTask(Task::Warning, warningMessage));
     emit addOutput(warningMessage, OutputFormat::ErrorMessage);
 }
 
