@@ -144,7 +144,7 @@ QWidget *FindInFiles::createConfigWidget()
     if (!m_configWidget) {
         m_configWidget = new QWidget;
         auto gridLayout = new QGridLayout(m_configWidget);
-        gridLayout->setMargin(0);
+        gridLayout->setContentsMargins(0, 0, 0, 0);
         m_configWidget->setLayout(gridLayout);
 
         int row = 0;
@@ -229,7 +229,7 @@ void FindInFiles::setDirectory(const FilePath &directory)
 
 void FindInFiles::setBaseDirectory(const FilePath &directory)
 {
-    m_directory->setBaseFileName(directory);
+    m_directory->setBaseDirectory(directory);
 }
 
 FilePath FindInFiles::directory() const
