@@ -45,7 +45,9 @@
 #include <tools/profile.h>
 #include <tools/settings.h>
 #include <tools/version.h>
+#include <tools/qttools.h>
 
+#include <QtCore/qbytearraylist.h>
 #include <QtCore/qcoreapplication.h>
 #include <QtCore/qdir.h>
 #include <QtCore/qdiriterator.h>
@@ -63,20 +65,8 @@ static QString qls(const char *s) { return QLatin1String(s); }
 
 static QStringList expectedArchs()
 {
-    return QStringList()
-            << QStringLiteral("arm64")
-            << QStringLiteral("armv5te")
-            << QStringLiteral("armv7a")
-            << QStringLiteral("mips")
-            << QStringLiteral("mips64")
-            << QStringLiteral("x86")
-            << QStringLiteral("x86_64");
-}
-
-
-static QString subProfileName(const QString &mainProfileName, const QString &arch)
-{
-    return mainProfileName + QLatin1Char('-') + arch;
+    return {QStringLiteral("arm64"), QStringLiteral("armv7a"),
+            QStringLiteral("x86"), QStringLiteral("x86_64")};
 }
 
 void setupSdk(qbs::Settings *settings, const QString &profileName, const QString &sdkDirPath)
@@ -105,10 +95,10 @@ static QString mapArch(const QString &androidName)
 }
 
 struct QtAndroidInfo {
-    bool isValid() const { return !arch.isEmpty(); }
+    bool isValid() const { return !archs.isEmpty(); }
 
     QString qmakePath;
-    QString arch;
+    QStringList archs;
     QString platform;
 };
 
@@ -122,19 +112,30 @@ static QtAndroidInfo getInfoForQtDir(const QString &qtDir)
     if (!qdevicepri.open(QIODevice::ReadOnly))
         return info;
     while (!qdevicepri.atEnd()) {
+        // For Qt < 5.14 use DEFAULT_ANDROID_TARGET_ARCH (which is the abi) to compute
+        // the architecture
+        // DEFAULT_ANDROID_ABIS doesn't exit
+        // For Qt >= 5.14:
+        // DEFAULT_ANDROID_TARGET_ARCH doesn't exist, use DEFAULT_ANDROID_ABIS to compute
+        // the architectures
         const QByteArray line = qdevicepri.readLine().simplified();
         const bool isArchLine = line.startsWith("DEFAULT_ANDROID_TARGET_ARCH");
+        const bool isAbisLine = line.startsWith("DEFAULT_ANDROID_ABIS");
         const bool isPlatformLine = line.startsWith("DEFAULT_ANDROID_PLATFORM");
-        if (!isArchLine && !isPlatformLine)
+        if (!isArchLine && !isPlatformLine && !isAbisLine)
             continue;
         const QList<QByteArray> elems = line.split('=');
         if (elems.size() != 2)
             continue;
         const QString rhs = QString::fromLatin1(elems.at(1).trimmed());
-        if (isArchLine)
-            info.arch = mapArch(rhs);
-        else
+        if (isArchLine) {
+            info.archs << mapArch(rhs);
+        } else if (isAbisLine) {
+            for (const QString &abi: rhs.split(QLatin1Char(' ')))
+                info.archs << mapArch(abi);
+        } else {
             info.platform = rhs;
+        }
     }
     return info;
 }
@@ -147,13 +148,16 @@ static QtInfoPerArch getQtAndroidInfo(const QString &qtSdkDir)
         return archs;
 
     QStringList qtDirs(qtSdkDir);
-    QDirIterator dit(qtSdkDir, QStringList() << QLatin1String("android_*"), QDir::Dirs);
+    const QStringList nameFilters{QStringLiteral("android_*"), QStringLiteral("android")};
+    QDirIterator dit(qtSdkDir, nameFilters, QDir::Dirs);
     while (dit.hasNext())
         qtDirs << dit.next();
-    for (auto it = qtDirs.cbegin(); it != qtDirs.cend(); ++it) {
-        const QtAndroidInfo info = getInfoForQtDir(*it);
-        if (info.isValid())
-            archs.insert(info.arch, info);
+    for (const auto &qtDir : qtDirs) {
+        const QtAndroidInfo info = getInfoForQtDir(qtDir);
+        if (info.isValid()) {
+            for (const QString &arch: info.archs)
+                archs.insert(arch, info);
+        }
     }
     return archs;
 }
@@ -181,6 +185,29 @@ static QString maximumPlatform(const QString &platform1, const QString &platform
     return prefix + QString::number(std::max(value1, value2));
 }
 
+static QString getToolchainType(const QString &ndkDirPath)
+{
+    QFile sourceProperties(ndkDirPath + qls("/source.properties"));
+    if (!sourceProperties.open(QIODevice::ReadOnly))
+        return QStringLiteral("gcc"); // <= r10
+    while (!sourceProperties.atEnd()) {
+        const QByteArray curLine = sourceProperties.readLine().simplified();
+        static const QByteArray prefix = "Pkg.Revision = ";
+        if (!curLine.startsWith(prefix))
+            continue;
+        qbs::Version ndkVersion = qbs::Version::fromString(
+                    QString::fromLatin1(curLine.mid(prefix.size())));
+        if (!ndkVersion.isValid()) {
+            qWarning("Unexpected format of NDK revision string in '%s'",
+                     qPrintable(sourceProperties.fileName()));
+            return QStringLiteral("clang");
+        }
+        return qls(ndkVersion.majorVersion() >= 18 ? "clang" : "gcc");
+    }
+    qWarning("No revision entry found in '%s'", qPrintable(sourceProperties.fileName()));
+    return QStringLiteral("clang");
+}
+
 static void setupNdk(qbs::Settings *settings, const QString &profileName, const QString &ndkDirPath,
                      const QString &qtSdkDirPath)
 {
@@ -194,40 +221,28 @@ static void setupNdk(qbs::Settings *settings, const QString &profileName, const
         mainProfile.setValue(qls("Android.ndk.ndkDir"), QDir::cleanPath(ndkDirPath));
         mainProfile.setValue(qls("Android.sdk.ndkDir"), QDir::cleanPath(ndkDirPath));
     }
-    mainProfile.setValue(qls("qbs.toolchain"), QStringList() << qls("gcc"));
+    mainProfile.setValue(qls("qbs.toolchainType"), getToolchainType(ndkDirPath));
     const QStringList archs = expectedArchs();
     const QtInfoPerArch infoPerArch = getQtAndroidInfo(qtSdkDirPath);
-    mainProfile.setValue(qls("qbs.architectures"), infoPerArch.empty()
-                         ? archs : QStringList(infoPerArch.keys()));
-    QStringList searchPaths;
+    const QStringList archsForProfile = infoPerArch.empty()
+            ? archs : QStringList(infoPerArch.keys());
+    if (archsForProfile.size() == 1)
+        mainProfile.setValue(qls("qbs.architecture"), archsForProfile.front());
+    else
+        mainProfile.setValue(qls("qbs.architectures"), archsForProfile);
+    QStringList qmakeFilePaths;
     QString platform;
     for (const QString &arch : archs) {
         const QtAndroidInfo qtAndroidInfo = infoPerArch.value(arch);
         if (!qtAndroidInfo.isValid())
             continue;
-        const QString subProName = subProfileName(profileName, arch);
-        const QString setupQtPath = qApp->applicationDirPath() + qls("/qbs-setup-qt");
-        QProcess setupQt;
-        setupQt.start(setupQtPath, QStringList({ qtAndroidInfo.qmakePath, subProName }));
-        if (!setupQt.waitForStarted()) {
-            throw ErrorInfo(Tr::tr("Setting up Qt profile failed: '%1' "
-                                   "could not be started.").arg(setupQtPath));
-        }
-        if (!setupQt.waitForFinished()) {
-            throw ErrorInfo(Tr::tr("Setting up Qt profile failed: Error running '%1' (%2)")
-                            .arg(setupQtPath, setupQt.errorString()));
-        }
-        if (setupQt.exitCode() != 0) {
-            throw ErrorInfo(Tr::tr("Setting up Qt profile failed: '%1' returned with "
-                                   "exit code %2.").arg(setupQtPath).arg(setupQt.exitCode()));
-        }
-        settings->sync();
-        qbs::Internal::TemporaryProfile p(subProName, settings);
-        searchPaths << p.p.value(qls("preferences.qbsSearchPaths")).toStringList();
+        qmakeFilePaths << qtAndroidInfo.qmakePath;
         platform = maximumPlatform(platform, qtAndroidInfo.platform);
     }
-    if (!searchPaths.empty())
-        mainProfile.setValue(qls("preferences.qbsSearchPaths"), searchPaths);
+    if (!qmakeFilePaths.empty()) {
+        qmakeFilePaths.removeDuplicates();
+        mainProfile.setValue(qls("moduleProviders.Qt.qmakeFilePaths"), qmakeFilePaths);
+    }
     if (!platform.isEmpty())
         mainProfile.setValue(qls("Android.ndk.platform"), platform);
 }
